/**
 * Base class for webservices delegate.
 * className: WebserviceDelegate
 *
 * @author Cedric Chantepie (cchantepie@corsaire.fr)
 */
// --- Constructors ---

/**
 * Sole constructor.
 */
function WebserviceDelegate() {
} // end of <init>

WebserviceDelegate.prototype = {
  // --- nsISupports implementation ---

  /**
   * Retrieve instance as specified interface if supported.
   */
  QueryInterface: function QueryInterface(aIID) {
    if (Components.interfaces.nsIClassInfo.equals(aIID) ||
        Components.interfaces.nsISupportsString.equals(aIID) ||
        Components.interfaces.nsISupportsWeakReference.equals(aIID) ||
        Components.interfaces.nsISupports.equals(aIID)) {
      
      return this;
    } // end of if
    
    else {
      throw 0x80004002; // Components.results.NS_NOINTERFACE;
    } // end of else
  }, // end of QueryInterface

  // --- nsIClassInfo implementation ---

  /**
   * Return count of implemented interfaces and 
   * there implemented interfaces.
   *
   * @param count (out) Count of implemented interfaces.
   * @return Array of implemented interfaces. 
   */
  getInterfaces: function getInterfaces(count) {
    count.value = 4;

    return [Components.interfaces.nsIClassInfo,
            Components.interfaces.nsISupportsString,
            Components.interfaces.nsISupportsWeakReference,
            Components.interfaces.nsISupports];

  }, // end of getInterfaces

  /**
   */
  getHelperForLanguage: function getHelperForLanguage(language) { 
    return null;
  }, // end of getHelperForLanguage

  /**
   */
  get contractID() { 
    return null; 
  }, // end of contractID getter

  /**
   * Return description of this class.
   */
  get classDescription() { 
    return "WebserviceDelegate"; 
  }, // end of classDescription

  /**
   */
  get classID() {
    return null; 
  }, // end of classID getter

  /**
   * Returns language in which is implemented this class.
   */
  get implementationLanguage() { 
    return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; 
  }, // end of implementationLanguage getter
  
  /**
   * Returns class flags.
   */
  get flags() {
    return Components.interfaces.nsIClassInfo.MAIN_THREAD_ONLY;
  }, // end of flags getter

  // --- nsISupportsString implementation ---

  /**
   * Returns string data about this instance.
   */
  get data() {
    return "[object " + this.classDescription + "]"; 
  }, // end of data getter

  /**
   * Returns a string representation of this instance.
   */
  toString: function toString() { 
    return this.data;
  }, // end of toString

  // --- Properties ---
  
  /**
   * delegatee: WebServiceProxy
   */
  delegatee: null,

  /**
   * Webservice response handler.
   */
  handler: null,

  // ---

  /**
   * Returns a plain js object from a proxy object from webservices.
   *
   * @param proxyObj
   */
  proxyToJSObject: function proxyToJSObject(proxyObj) {
    var jsObject = new Object();
    
    for (member in proxyObj) {
      if (!(proxyObj[member] instanceof Function)) {
	jsObject[member] = proxyObj[member];
      } // end of if
    } // end of for

    // ---

    return jsObject;
  } // end of proxyToJSObject

}; // end of WebserviceDelegate prototype
